# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/ImportCameras.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ImportCameras(object):
    def setupUi(self, ImportCameras):
        ImportCameras.setObjectName("ImportCameras")
        ImportCameras.resize(400, 249)
        self.verticalLayout = QtWidgets.QVBoxLayout(ImportCameras)
        self.verticalLayout.setObjectName("verticalLayout")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.buttonBox = QtWidgets.QDialogButtonBox(ImportCameras)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(ImportCameras)
        self.buttonBox.accepted.connect(ImportCameras.accept)
        self.buttonBox.rejected.connect(ImportCameras.reject)
        QtCore.QMetaObject.connectSlotsByName(ImportCameras)

    def retranslateUi(self, ImportCameras):
        _translate = QtCore.QCoreApplication.translate
        ImportCameras.setWindowTitle(_translate("ImportCameras", "Import cameras ..."))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/PositionControl.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_PositionControl(object):
    def setupUi(self, PositionControl):
        PositionControl.setObjectName("PositionControl")
        PositionControl.resize(415, 495)
        self.verticalLayout = QtWidgets.QVBoxLayout(PositionControl)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gb_pan = QtWidgets.QGroupBox(PositionControl)
        self.gb_pan.setAlignment(QtCore.Qt.AlignCenter)
        self.gb_pan.setFlat(False)
        self.gb_pan.setCheckable(False)
        self.gb_pan.setObjectName("gb_pan")
        self.gridLayout = QtWidgets.QGridLayout(self.gb_pan)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.lb_min_pan = QtWidgets.QLabel(self.gb_pan)
        self.lb_min_pan.setObjectName("lb_min_pan")
        self.gridLayout.addWidget(self.lb_min_pan, 0, 0, 1, 1)
        self.sl_pos_pan = QtWidgets.QSlider(self.gb_pan)
        self.sl_pos_pan.setOrientation(QtCore.Qt.Horizontal)
        self.sl_pos_pan.setObjectName("sl_pos_pan")
        self.gridLayout.addWidget(self.sl_pos_pan, 1, 0, 1, 6)
        self.lb_max_pan = QtWidgets.QLabel(self.gb_pan)
        self.lb_max_pan.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lb_max_pan.setObjectName("lb_max_pan")
        self.gridLayout.addWidget(self.lb_max_pan, 0, 5, 1, 1)
        self.sb_move_pan = QtWidgets.QDoubleSpinBox(self.gb_pan)
        self.sb_move_pan.setObjectName("sb_move_pan")
        self.gridLayout.addWidget(self.sb_move_pan, 0, 3, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 0, 4, 1, 1)
        self.label = QtWidgets.QLabel(self.gb_pan)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 2, 1, 1)
        self.verticalLayout.addWidget(self.gb_pan)
        self.gb_tilt = QtWidgets.QGroupBox(PositionControl)
        self.gb_tilt.setAlignment(QtCore.Qt.AlignCenter)
        self.gb_tilt.setObjectName("gb_tilt")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.gb_tilt)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem2 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem2, 0, 1, 1, 1)
        self.sb_move_tilt = QtWidgets.QDoubleSpinBox(self.gb_tilt)
        self.sb_move_tilt.setObjectName("sb_move_tilt")
        self.gridLayout_2.addWidget(self.sb_move_tilt, 0, 3, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.gb_tilt)
        self.label_4.setObjectName("label_4")
        self.gridLayout_2.addWidget(self.label_4, 0, 2, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem3, 0, 4, 1, 1)
        self.lb_min_tilt = QtWidgets.QLabel(self.gb_tilt)
        self.lb_min_tilt.setObjectName("lb_min_tilt")
        self.gridLayout_2.addWidget(self.lb_min_tilt, 0, 0, 1, 1)
        self.sl_pos_tilt = QtWidgets.QSlider(self.gb_tilt)
        self.sl_pos_tilt.setOrientation(QtCore.Qt.Horizontal)
        self.sl_pos_tilt.setObjectName("sl_pos_tilt")
        self.gridLayout_2.addWidget(self.sl_pos_tilt, 1, 0, 1, 6)
        self.lb_max_tilt = QtWidgets.QLabel(self.gb_tilt)
        self.lb_max_tilt.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lb_max_tilt.setObjectName("lb_max_tilt")
        self.gridLayout_2.addWidget(self.lb_max_tilt, 0, 5, 1, 1)
        self.verticalLayout.addWidget(self.gb_tilt)
        self.gb_zoom = QtWidgets.QGroupBox(PositionControl)
        self.gb_zoom.setAlignment(QtCore.Qt.AlignCenter)
        self.gb_zoom.setObjectName("gb_zoom")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.gb_zoom)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_6 = QtWidgets.QLabel(self.gb_zoom)
        self.label_6.setObjectName("label_6")
        self.gridLayout_3.addWidget(self.label_6, 0, 2, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem4, 0, 4, 1, 1)
        self.lb_max_zoom = QtWidgets.QLabel(self.gb_zoom)
        self.lb_max_zoom.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lb_max_zoom.setObjectName("lb_max_zoom")
        self.gridLayout_3.addWidget(self.lb_max_zoom, 0, 5, 1, 1)
        self.sl_pos_zoom = QtWidgets.QSlider(self.gb_zoom)
        self.sl_pos_zoom.setOrientation(QtCore.Qt.Horizontal)
        self.sl_pos_zoom.setObjectName("sl_pos_zoom")
        self.gridLayout_3.addWidget(self.sl_pos_zoom, 1, 0, 1, 6)
        self.lb_min_zoom = QtWidgets.QLabel(self.gb_zoom)
        self.lb_min_zoom.setObjectName("lb_min_zoom")
        self.gridLayout_3.addWidget(self.lb_min_zoom, 0, 0, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem5, 0, 1, 1, 1)
        self.sb_move_zoom = QtWidgets.QSpinBox(self.gb_zoom)
        self.sb_move_zoom.setObjectName("sb_move_zoom")
        self.gridLayout_3.addWidget(self.sb_move_zoom, 0, 3, 1, 1)
        self.verticalLayout.addWidget(self.gb_zoom)
        self.gb_focus = QtWidgets.QGroupBox(PositionControl)
        self.gb_focus.setAlignment(QtCore.Qt.AlignCenter)
        self.gb_focus.setCheckable(True)
        self.gb_focus.setObjectName("gb_focus")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.gb_focus)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_8 = QtWidgets.QLabel(self.gb_focus)
        self.label_8.setObjectName("label_8")
        self.gridLayout_4.addWidget(self.label_8, 1, 2, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem6, 1, 4, 1, 1)
        self.lb_max_focus = QtWidgets.QLabel(self.gb_focus)
        self.lb_max_focus.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lb_max_focus.setObjectName("lb_max_focus")
        self.gridLayout_4.addWidget(self.lb_max_focus, 1, 5, 1, 1)
        self.sl_pos_focus = QtWidgets.QSlider(self.gb_focus)
        self.sl_pos_focus.setOrientation(QtCore.Qt.Horizontal)
        self.sl_pos_focus.setObjectName("sl_pos_focus")
        self.gridLayout_4.addWidget(self.sl_pos_focus, 2, 0, 1, 6)
        self.lb_min_focus = QtWidgets.QLabel(self.gb_focus)
        self.lb_min_focus.setObjectName("lb_min_focus")
        self.gridLayout_4.addWidget(self.lb_min_focus, 1, 0, 1, 1)
        spacerItem7 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem7, 1, 1, 1, 1)
        self.sb_move_focus = QtWidgets.QSpinBox(self.gb_focus)
        self.sb_move_focus.setObjectName("sb_move_focus")
        self.gridLayout_4.addWidget(self.sb_move_focus, 1, 3, 1, 1)
        self.verticalLayout.addWidget(self.gb_focus)
        spacerItem8 = QtWidgets.QSpacerItem(20, 150, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem8)

        self.retranslateUi(PositionControl)
        QtCore.QMetaObject.connectSlotsByName(PositionControl)

    def retranslateUi(self, PositionControl):
        _translate = QtCore.QCoreApplication.translate
        PositionControl.setWindowTitle(_translate("PositionControl", "Position Control"))
        self.gb_pan.setTitle(_translate("PositionControl", "Pan: 55.5"))
        self.lb_min_pan.setText(_translate("PositionControl", "-100"))
        self.lb_max_pan.setText(_translate("PositionControl", "100"))
        self.label.setText(_translate("PositionControl", "Move to:"))
        self.gb_tilt.setTitle(_translate("PositionControl", "Tilt: 101.2"))
        self.label_4.setText(_translate("PositionControl", "Move to:"))
        self.lb_min_tilt.setText(_translate("PositionControl", "-100"))
        self.lb_max_tilt.setText(_translate("PositionControl", "100"))
        self.gb_zoom.setTitle(_translate("PositionControl", "Zoom: 150"))
        self.label_6.setText(_translate("PositionControl", "Move to:"))
        self.lb_max_zoom.setText(_translate("PositionControl", "100"))
        self.lb_min_zoom.setText(_translate("PositionControl", "-100"))
        self.gb_focus.setTitle(_translate("PositionControl", "Focus: "))
        self.label_8.setText(_translate("PositionControl", "Move to:"))
        self.lb_max_focus.setText(_translate("PositionControl", "100"))
        self.lb_min_focus.setText(_translate("PositionControl", "-100"))




#!/usr/bin/python3
import io
import os
from petra_camera.version import __version__

from setuptools import setup, find_packages

# Package meta-data.
NAME = 'petra_camera'
DESCRIPTION = 'Simple viewer for cameras, used at PETRA III source'
EMAIL = 'yury.matveev@desy.de'
AUTHOR = 'Yury Matveyev'
REQUIRES_PYTHON = '>=3.7'

# What packages are required for this module to be executed?
REQUIRED = ['pyqtgraph', 'psutil', 'numpy', 'scipy',
]

EXTRA_REQUIRED = {'LAMBDA': ['watchdog'],
                  'PEAK': ['scikit-image']
                  }

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
here = os.path.abspath(os.path.dirname(__file__))
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except IOError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
about = {}
about['__version__'] = __version__

# Where the magic happens:
setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    packages=find_packages(),
    package_dir={'petra_camera': 'petra_camera',},
    package_data={'petra_camera': ['petra_camera/*.py', 'petra_camera/*.xml' ],},
    install_requires=REQUIRED,
    extras_require=EXTRA_REQUIRED,
    include_package_data=True,
    license='GPLv3',
    entry_points={'console_scripts': ['petra_camera = petra_camera:main',],},
    scripts=['petra_camera/petra_camera.sh'],
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Development Status :: 5 - Production/Stable'
    ],
)



# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------

"""Dummy 2D data generator.
"""
import threading
import time
import requests
from random import random
import io
from PIL import Image
import numpy as np
import logging

from threading import Thread

from petra_camera.devices.base_camera import BaseCamera
from petra_camera.main_window import APP_NAME

logger = logging.getLogger(APP_NAME)


# ----------------------------------------------------------------------
class PetraStatus(BaseCamera):
    """
    """
    FRAME_W = 800
    FRAME_H = 600

    _settings_map = {'max_width': ('self', 'FRAME_W'),
                     'max_height': ('self', 'FRAME_H')}

    visible_layouts = ('FPS')

    # ----------------------------------------------------------------------
    def __init__(self, settings):
        super(PetraStatus, self).__init__(settings)

        self._fps = self.get_settings('FPS', int)
        if self._fps == 0:
            self._fps = 1

        self._picture_thread = Thread(target=self._readout_frame)
        self._generator_thread_working = False
        self._stop_picture_thread = threading.Event()
        self._run_acquisition = threading.Event()
        self._picture_thread.start()

    # ----------------------------------------------------------------------
    def close_camera(self):

        self._stop_picture_thread.set()
        while self._generator_thread_working:
            time.sleep(1/self._fps)

    # ----------------------------------------------------------------------
    def _readout_frame(self):

        while not self._stop_picture_thread.is_set():

            time.sleep(1 / self._fps)

            if self._run_acquisition.is_set():
                ans = requests.get(f'https://winweb.desy.de/mca/accstatus/infoscreen/petra_status_800.png?{random()}')
                if ans.status_code == 200:
                    picture_stream = io.BytesIO(ans.content)
                    picture = Image.open(picture_stream)
                    self._last_frame = np.rot90(np.asarray(picture, dtype=np.int32), 1)[::-1, :]
                    self._new_frame_flag = True
                    logger.debug(f"{self._my_name} new frame")

        self._generator_thread_working = False

    # ----------------------------------------------------------------------
    def _start_acquisition(self):

        logger.debug(f"{self._my_name} starting thread")

        self._run_acquisition.set()
        return True

    # ----------------------------------------------------------------------
    def stop_acquisition(self):
        self._run_acquisition.clear()

    # ----------------------------------------------------------------------
    def is_running(self):

        return self._run_acquisition.is_set()

    # ----------------------------------------------------------------------
    def get_settings(self, option, cast, do_rotate=True, do_log=True):

        if option in ['FPSmax', 'max_width', 'max_height']:

            logger.debug(f'{self._my_name}: setting {cast.__name__}({option}) requested')

            if option == 'FPSmax':
                return 200
            elif option == 'max_width':
                return self.FRAME_W
            elif option == 'max_height':
                return self.FRAME_H
        else:
            return super(PetraStatus, self).get_settings(option, cast, do_rotate, do_log)

    # ----------------------------------------------------------------------
    def save_settings(self, option, value):

        if option == 'FPS':

            logger.debug(f'{self._my_name}: setting {option}: new value {value}')
            self._fps = value

        super(PetraStatus, self).save_settings(option, value)


# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------

"""
"""
import threading

try:
    import PyTango
except ImportError:
    pass
from contextlib import contextmanager
import logging


from PyQt5 import QtCore

from petra_camera.widgets.base_widget import BaseWidget
from petra_camera.main_window import APP_NAME
from petra_camera.gui.PositionControl_ui import Ui_PositionControl

WIDGET_NAME = 'PositionControl'

logger = logging.getLogger(APP_NAME)

REFRESH_PERIOD = 1


# ----------------------------------------------------------------------
class PositionControl(BaseWidget):
    """
    """
    # ----------------------------------------------------------------------
    def __init__(self, parent):
        """
        """
        super(PositionControl, self).__init__(parent)

        self._ui = Ui_PositionControl()
        self._ui.setupUi(self)

        self.setup_limits()

        # to prevent double code run
        self._my_mutex = QtCore.QMutex()
        self._stop_position_reader = threading.Event()

        for param in ['pan', 'tilt', 'focus', 'zoom']:
            getattr(self._ui, f'sb_move_{param}').editingFinished.connect(lambda x=param: self.move_to_sb(x))

        for param in ['pan', 'tilt', 'focus', 'zoom']:
            getattr(self._ui, f'sl_pos_{param}').valueChanged.connect(lambda value, x=param: self.move_to_sl(x, value))

        self._position_reader = PositionReader(self._camera_device, self._stop_position_reader)
        self._position_reader.position_ready.connect(self.display_position)
        self._position_reader.start()

    # ----------------------------------------------------------------------
    def setup_limits(self):
        with self.block_signals():
            for limit in ['min', 'max']:
                for cast_type, elements in zip([float, int], [['pan', 'tilt'], ['focus', 'zoom']]):
                    for param in elements:
                        value = self._camera_device.get_settings(f'{limit}_{param}', cast_type)
                        getattr(self._ui, f'lb_{limit}_{param}').setText(f'{value:.{0 if cast_type==int else 2}f}')
                        if limit == 'min':
                            getattr(self._ui, f'sl_pos_{param}').setMinimum(value)
                            getattr(self._ui, f'sb_move_{param}').setMinimum(value)
                        else:
                            getattr(self._ui, f'sl_pos_{param}').setMaximum(value)
                            getattr(self._ui, f'sb_move_{param}').setMaximum(value)

    # ----------------------------------------------------------------------
    @contextmanager
    def block_signals(self):
        for param in ['pan', 'tilt', 'focus', 'zoom']:
            getattr(self._ui, f'sb_move_{param}').blockSignals(True)
            getattr(self._ui, f'sl_pos_{param}').blockSignals(True)

        yield

        for param in ['pan', 'tilt', 'focus', 'zoom']:
            getattr(self._ui, f'sb_move_{param}').blockSignals(False)
            getattr(self._ui, f'sl_pos_{param}').blockSignals(False)

    # ----------------------------------------------------------------------
    def display_position(self):
        with QtCore.QMutexLocker(self._my_mutex):
            with self.block_signals():
                for params, decimals in zip([['pan', 'tilt'], ['zoom', 'focus']], [2, 0]):
                    for param in params:
                        for ui in ['sl_pos', 'sb_move']:
                            if not getattr(self._ui, f'{ui}_{param}').hasFocus():
                                value = getattr(self._position_reader, f'{param}')
                                getattr(self._ui, f'{ui}_{param}').setValue(value)
                                getattr(self._ui, f'gb_{param}').setTitle(f'{param.capitalize()}: {value:.{decimals}f}')

    # ----------------------------------------------------------------------
    def move_to_sb(self, param):
        self._camera_device.save_settings(param, getattr(self._ui, f'sb_move_{param}').value())

    # ----------------------------------------------------------------------
    def move_to_sl(self, param, value):
        self._camera_device.save_settings(param, value)


# ----------------------------------------------------------------------
class PositionReader(QtCore.QThread):

    position_ready = QtCore.pyqtSignal()

    def __init__(self, camera_device, stop_request):
        super().__init__()
        self._camera_device = camera_device
        self._stop_requested = stop_request

        self.pan = None
        self.tilt = None
        self.focus = None
        self.zoom = None

    # ----------------------------------------------------------------------
    def run(self):
        while not self._stop_requested.is_set():

            self.pan = self._camera_device.get_settings('pan', float)
            self.tilt = self._camera_device.get_settings('tilt', float)
            self.focus = self._camera_device.get_settings('focus', int)
            self.zoom = self._camera_device.get_settings('zoom', int)

            self.position_ready.emit()

            QtCore.QThread.msleep(REFRESH_PERIOD*1000)